from typing import Annotated
import dbus.service
from fastapi import Depends, HTTPException, Request
import dbus

from anime import dtos


def _get_dbus_session(request: Request) -> dbus.SessionBus:
    return request.app.state.dbus_session


class DbusService:
    def __init__(
        self,
        dbus_session: Annotated[dbus.SessionBus, Depends(_get_dbus_session)],
    ) -> None:
        self.session = dbus_session

    def _get_player(self, name: str) -> dbus.service.Object:
        if name:
            try:
                return self.session.get_object(
                    f"org.mpris.MediaPlayer2.{name}",
                    "/org/mpris/MediaPlayer2",
                )
            except dbus.exceptions.DBusException as exc:
                raise HTTPException(
                    status_code=400,
                    detail=f"Player {name} is not available",
                ) from exc

        for name in self.session.list_names():
            if not name.startswith("org.mpris.MediaPlayer2."):
                continue
            return self.session.get_object(name, "/org/mpris/MediaPlayer2")
        raise HTTPException(status_code=400, detail="No player is available")

    def _resolve_name(self, name: str | None = None) -> str:
        if name is None:
            all_players = [
                name
                for name in self.session.list_names()
                if name.startswith("org.mpris.MediaPlayer2.")
            ]
            if not all_players:
                raise HTTPException(
                    status_code=400,
                    detail="No player is available",
                )
            return all_players[0].removeprefix("org.mpris.MediaPlayer2.")
        if name:
            return name

    def list_players(self) -> list[dtos.PlayerInfo]:
        state = []
        for name in self.session.list_names():
            if not name.startswith("org.mpris.MediaPlayer2."):
                continue
            player = self.session.get_object(name, "/org/mpris/MediaPlayer2")
            status = player.Get(
                "org.mpris.MediaPlayer2.Player",
                "PlaybackStatus",
                dbus_interface="org.freedesktop.DBus.Properties",
            )
            metadata = player.Get(
                "org.mpris.MediaPlayer2.Player",
                "Metadata",
                dbus_interface="org.freedesktop.DBus.Properties",
            )
            can_seek = player.Get(
                "org.mpris.MediaPlayer2.Player",
                "CanSeek",
                dbus_interface="org.freedesktop.DBus.Properties",
            )
            track_len = metadata.get("mpris:length")
            progress = None
            try:
                position = player.Get(
                    "org.mpris.MediaPlayer2.Player",
                    "Position",
                    dbus_interface="org.freedesktop.DBus.Properties",
                )
                if position is not None and track_len is not None:
                    progress = ((position * 100) // track_len)
            except dbus.exceptions.DBusException:
                pass
            artists = metadata.get("xesam:artist")
            album = metadata.get("xesam:album")
            title = metadata.get("xesam:title")
            state.append(
                dtos.PlayerInfo(
                    name=name.removeprefix("org.mpris.MediaPlayer2."),
                    playing=status.lower() == "playing",
                    can_seek=bool(can_seek),
                    artists=[str(artist) for artist in artists] if artists else [],
                    title=str(title) if title else None,
                    album=str(album) if album else None,
                    progress=progress,
                )
            )
        return state

    def seek(
        self,
        offset: int,
        name: str | None = None,
    ) -> None:
        resolved_name = self._resolve_name(name)
        player = self._get_player(resolved_name)
        try:
            player.Seek(
                offset * 1_000_000,
                dbus_interface="org.mpris.MediaPlayer2.Player",
            )
        except dbus.exceptions.DBusException as exc:
            raise HTTPException(
                status_code=400,
                detail=f"Player {resolved_name} is not seekable",
            ) from exc

    def play_pause(self, name: str | None = None) -> None:
        resolved_name = self._resolve_name(name)
        player = self._get_player(resolved_name)
        try:
            player.PlayPause(dbus_interface="org.mpris.MediaPlayer2.Player")
        except dbus.exceptions.DBusException as exc:
            raise HTTPException(
                status_code=400,
                detail=f"Player {resolved_name} cannot be controlled",
            ) from exc
